==> jungle-1.0.tar.gz: jungle-1.0/jungle/apps/timereport/models.py <==
from django.db import models
from jungle.ref.models import WeekDay
from jungle.userprofile.models import Profile
from django.utils.translation import gettext as _
#from jungle.templatetags.jungle_tags import ugettext as _
from django.contrib.auth.models import User
from datetime import *
from django.conf import settings
from jungle.ticket.models import Project
from jungle.db import get_records
import decimal
import datetime

class TimeOutMissingTimeIn(Exception): pass

                       

# class TransitManager(models.Manager):
#     def get_query_set(self):
#         from jungle.middleware.thread_local import get_current_user

#         self.user = get_current_user()
#         original = super(TransitManager, self).get_query_set()

#         if self.user and self.user.is_authenticated() and self.user.is_staff:
#             return original
#         elif self.user and self.user.is_authenticated():
#             return original.filter(user=self.user)
#         else:
#             return super(TransitManager, self).none()

STATUS_CODES = (
    (0, _('inactive')),
    (1, _('read by terminal')),
    (2, _('manually inserted')),
    (3, _('automatically inserted')),
)

DIRECTION = (
  ('0', _('No transit')),  #eg.: holiday
  ('1', _('In')),
  ('2', _('Out')),
  ('T', _('Transit')),
    )
TYPES = (
    ('1', 'Transito locale presenze accessi'),
#    ('2', 'Transito accesso remoto'),
#    ('3', 'Transito rifiutato da controllo accessi'),
    ('4', 'Transito con causale'),  # not really used. Any transit haas a reason
#    ('5', 'Transito con causale menu a tentina'),
    ('7', 'Consumo mensa'),
#    ('9', 'Transito inserito da seriale'),
#    ('A', 'Transito virtale inserito da terminale'),
    ('B', 'Prenotazione mensa'),
    )
ODV_CHOICES = (
    (1,'ingresso mattina'),
    (2,'uscita mattina'),
    (3,'ingresso pomeriggio'),
    (4,'uscita pomeriggio'),
#    (10,'extra'),
    (11,'extra ingresso'),
    (12,'extra uscita'),
    )
##### Transit
class Transit(models.Model):
    """date_reference is in case the end of the turn passes midnight: exiting at
       2 a.m. should set date_reference the day before. That will easy all operations
    """
#    objects = TransitManager()
    objects = models.Manager()
    cron_objects = models.Manager()
    ## Campi generici
    date_create = models.DateTimeField (auto_now_add=True,null=False)
    date_last_modify = models.DateTimeField (auto_now=True,null=False)
    status = models.IntegerField (choices=STATUS_CODES, default=2)

    ## ForeignKey
    reason = models.ForeignKey('Reason', blank=True )
    user = models.ForeignKey(User, blank=True)

    ## Campi
    date_reference=models.DateField()
    date_time = models.DateTimeField (
        verbose_name=_('Datetime of transit'),
    )
    direction = models.CharField(
        max_length=1,
        verbose_name=_('Direction of transit'),
        choices=DIRECTION,
        )
    type = models.CharField(max_length=1, default=1,choices=TYPES)
    
    # FIXME : il profilo e' memorizzato come Integer.
    badge_num = models.CharField(max_length=12, null=True, blank=True) 
    badge_reader = models.ForeignKey('BadgeReader',blank=True, null=True)
    edited_by = models.ForeignKey(User, null=True, blank=True,
                                  related_name='edited')
    profile = models.ForeignKey('TimetableProfile')
    odv = models.IntegerField(null=True, blank=True, choices=ODV_CHOICES)
    approved = models.IntegerField(null=True, blank=True)
    iter = models.ForeignKey('Iter', null=True, blank=True)
    
    def __cmp__(self, other):
        return cmp(self.date_time, other.date_time)

    @classmethod
    def get_worked(self, user, date):
        SQL = """SELECT date_time FROM timereport_transit t
        LEFT JOIN timereport_reason r
        ON (t.reason_id = r.code)
        WHERE t.user_id = %s AND r.worked = %s AND date_reference = %s
        ORDER BY date_time DESC"""
        transits = get_records(SQL, [user.id, 'w', date],
                               "%(date_time)s")

        if len(transits) % 2:
            return None  ## Should I raise an error?
        else:
            total = timedelta()
            for i in range(0, len(transits), 2):
                total += transits[i].date_time - transits[i+1].date_time
            return total

    class Admin: pass

    class Meta:
        permissions = (
            ('add_others_transit', "can change other's people transits"),
            )
#        verbose_name = _('Transit')
#        verbose_name_plural = _('Transits')
         
    def __unicode__(self):
        return u"%s %s" % (self.user, self.date_time.strftime('%d/%m/%Y - %H:%M'))


####### BadgeReader
STATUS_CODES = (
    (0, 'inactive'),
    (1, 'active'),
    )

class BadgeReader(models.Model):
    ## Campi generici
    id = models.IntegerField(primary_key=True)
    date_create = models.DateTimeField (auto_now_add=True,null=False)
    date_last_modify = models.DateTimeField (auto_now=True,null=False)
    status = models.IntegerField (choices=STATUS_CODES, default=1)

    location = models.CharField(max_length=20)
    ip = models.CharField(max_length=15)
    user = models.CharField(max_length=15)
    password = models.CharField(max_length=15)

    def __unicode__(self):
        return unicode(self.location)
    
    class Admin:
        list_display = ('location', 'ip','user','password')

    class Meta:
        pass
#        verbose_name = _('Badge Reader')
#        verbose_name_plural = _('Badge Readers')
    
##### REASON (in italian: 'causale')
WORKED_CHOICES = (
    ('w',_('worked')),
    ('n',_('not worked'))
  )    
class Reason(models.Model):

    """this explains the reason of the transit. A 'default' code must be
    present. May be any locally accepted reason eg.: permit, permit due to
    medical visit... sole requirement is that worked indicate if that should
    be considered when building the total worked hours in a day.
    """
    ## Campi generici
    date_create = models.DateTimeField (auto_now_add=True,null=True)
    date_last_modify = models.DateTimeField (auto_now=True,null=True)
    status = models.IntegerField (choices=STATUS_CODES, default=1)

    ##
    code = models.CharField(max_length=7, null=True, blank=True, primary_key=True, unique=True)
    description = models.CharField(max_length=40)
    worked = models.CharField(max_length=1, choices=WORKED_CHOICES)
    column = models.CharField(max_length=7,
                             help_text=_("column in which to sum up"))  
    
    def __unicode__(self):
        return unicode(self.description) 

    class Admin: pass
#     class Meta:
#         verbose_name = _('Reason')
#         verbose_name_plural = _('Reasons')
    

class Iter(models.Model):
    ## generic fields
    date_create = models.DateTimeField (auto_now_add=True,null=False)
    date_last_modify = models.DateTimeField (auto_now=True,null=False)
    status = models.IntegerField (choices=STATUS_CODES, default=2)

    ## 'proper' fields
    date_start = models.DateField()  # mainly for 'fast_search'
    date_end = models.DateField()  # mainly for 'fast_search'
    user = models.ForeignKey(User)  # strictly unecessary and redoundant...
    description = models.CharField(max_length=100)
    date_approved1 = models.DateField(null=True)
    approved1_by = models.ForeignKey(User, null=True,
                                     related_name = 'approved1')
                                     
    date_approved2 = models.DateField(null=True)
    approved2_by = models.ForeignKey(User,null=True,
                                     related_name = 'approved2')
    reason = models.ForeignKey(Reason)

    def __unicode__(self):
        return u"%s - %s" % (self.date_start.strftime("%m/%d/%Y"),
                            self.description[:30])
        
    class Admin: pass
    

##### PROFILO CARTELLINO 

STATUS_CODES = (
    ( 1, 'Active'),
    ( 0, 'Inactive')
)

class TimetableProfile (models.Model):
    """each employee must have a TimetableProfile that describes how the TimeRegister
    should be taken into account.
    """
    ## Campi generici
    date_create = models.DateTimeField (auto_now_add=True,)
    date_last_modify = models.DateTimeField (auto_now=True,)
    status = models.IntegerField (choices=STATUS_CODES,)
    ##
    name = models.CharField(max_length=30, unique=True)
    only_presence = models.BooleanField(default=False, null=False)
    delta_advance  = models.IntegerField (verbose_name='Allowed advance flexibility in minutes')
    delta_delay = models.IntegerField (verbose_name='Allowed delay flexibility in minutes')
    hours = {}
    ProfileDayDetails = {}

    def __unicode__(self):
        return unicode(self.name)

    @classmethod
    def get_details(cls, filter={}):
        """
        get all profiles and return a dict that holds how many entries for each day
        are expected
        ProfileDayDetails = {
          '1' : {                               # profile N. 1: worker
            0 : {},                             # day 0 Sun -> no entries
            1 : {1:timedetail,3:timedetail},    # day 1 Mon -> 2 entries, 2 exit
            2 : {1:timedetail},                 # day 2 Tue -> 1 entry, 1 exit
                  ...
                  }
          '2' : ...                             # profile N. 2
          }
        """
        from time_display import MyDict
        #from inspect import isclass
        if not cls.ProfileDayDetails:
            Profiles = cls.objects.all().select_related().filter(**filter)
            for p in Profiles:
                DayEntry = MyDict()
                for i in range(1,8):   # isoweekdays: 1 -> 7
                    details = p.timetableprofiledetail_set.filter(isoweekday=i)
                    for x in details:
                        DayEntry[i][x.odv] = x
                        
                cls.ProfileDayDetails[p.id] = DayEntry

        return cls.ProfileDayDetails
        
    @classmethod
    def get_deltas(cls):
        """return a dict with deltas admitted for each profile"""
        deltas = {}
        for i in cls.objects.all():
            deltas[i.id] = (i.delta_advance, i.delta_delay)

        return deltas
    def get_day_hours(self, day):
        if isinstance(day, date):
            day = day.isoweekday()

        tot_seconds = 0
        for detail in self.get_my_details()[day].values():
            if detail:
                tot_seconds += detail.duration
        return decimal.Decimal(tot_seconds)/3600

    def get_total_working_seconds(self, isoweekday_id):
        ## in seconds since I need to compare it to datetime.timedelta
        
        if self.id in self.hours:
            return self.hours[self.id][isoweekday_id]
        
        details = self.timetableprofiledetail_set.all()
        D = {}
        for j in range(1,8): D[j] = 0
        
        for d in details.order_by('isoweekday_id'):
            D[d.isoweekday_id] += d.duration  # hours with 2 decimals

        self.hours[self.id] = D

        return D[isoweekday_id]
            
    def get_my_details(self, filter={}):
        return self.get_details(filter={'id':self.id})[self.id]

    def save(self):
        self.hours = {}
        self.ProfileDayDetails = {}
        super(TimetableProfile, self).save()

    class Admin: pass
    
    class Meta:
        verbose_name = 'Orario cartellino (profilo)'
        verbose_name_plural = 'Profili Orari'
    
    
class TimetableProfileDetail (models.Model):
    ## Campi generici
    date_create = models.DateTimeField (auto_now_add=True,)
    date_last_modify = models.DateTimeField (auto_now=True,)
    status = models.IntegerField (choices=STATUS_CODES,default=1)

    ## ForeignKey
    profile = models.ForeignKey (
        TimetableProfile,
        verbose_name='Related profile',
    )
    isoweekday = models.ForeignKey (
        WeekDay,
        verbose_name='Day',
        to_field='isoweekday',
    )
    ## Campi
    time = models.TimeField (
        verbose_name='Time In',
    )
    #direction = models.IntegerField(max_length=1, null=True, blank=True,choices=((0, 'In'), (1, 'Out')))
    duration = models.IntegerField()
    ## odv in general is 0,1,2,3: 0,2 way in, 1,3 way out
    odv = models.IntegerField()

    def get_time_out(self):
        ### FIXME: does not work for nighty turns
        if not self.duration:
            return None
        secs = time2sec(self.time) + self.duration
        return sec2time(secs)

    def set_time_out(self, time):
        #import pdb; pdb.set_trace()
        ### FIXME: does not work for nighty turns
        if not self.time:
            raise TimeOutMissingTimeIn
        
        duration = timediff(self.time, time).seconds
        self.duration = duration

    time_out = property(get_time_out, set_time_out)

    def __unicode__(self):
        return u"%s: %s" % (self.isoweekday, self.time and self.time.strftime('%H:%M'))
                     
    class Admin:
        pass

    class Meta:
        verbose_name = 'Dettagli Cartellino'
        verbose_name_plural = 'Dettagli Cartellino'
        unique_together=(('profile', 'isoweekday', 'odv'),)

class Final(models.Model):
    ## Campi generici
    date_create = models.DateTimeField (auto_now_add=True,)
    date_last_modify = models.DateTimeField (auto_now=True,)
    status = models.IntegerField (choices=STATUS_CODES,default=1)

    ##
    date = models.DateField()
    user = models.ForeignKey(User)
    theoric = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    worked = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    extra = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    missing = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    holiday = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    fesse = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    illness = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    notes = models.CharField(max_length=100)
    use_extra = models.BooleanField()

class Report(models.Model):
    #custom = ReportManager()
    ## Campi generici
    date_create = models.DateTimeField (auto_now_add=True,)
    date_last_modify = models.DateTimeField (auto_now=True,)
    status = models.IntegerField (choices=STATUS_CODES,default=1)
    ##
    user = models.ForeignKey(User)
    date = models.DateField()
    job = models.ForeignKey(Project)
    qty = models.IntegerField()
    unit = models.CharField(max_length=10)
    description = models.CharField(max_length=200)

    def hours(self):
        decimal.getcontext().prec = 3  # non funziona!
        return decimal.Decimal(self.qty)/100

    def __unicode__(self):
        return u"%s - %s - %s" % (self.user, self.date, self.job)

    def Date(self):
        return self.date.strftime("%Y/%m/%d")
    
    @classmethod
    def get_jobs(self, user_id, days, job_id = None):
        """return jobs 'used' in the last 'days' days"""
        date = datetime.date.today() - datetime.timedelta(days=days)
        attrs = [user_id, date]
        SQL = """SELECT DISTINCT job_id, tp.name
                 FROM timereport_report LEFT JOIN ticket_project tp
                 ON (job_id = tp.id)
                 WHERE user_id = %s AND date >= %s"""
        if job_id:
            SQL += " AND job_id = %s"
            attrs += [job_id]
        return get_records(SQL, attrs, "job: %(job_id)s")

    @classmethod
    def get_allocated(self, user_id, date):
        """return timedelta of hours in a day attributed by a user"""
        tot = datetime.timedelta()
        for e in self.objects.filter(user=user_id, date=date, qty__gt=0):
            h, minutes = divmod(e.qty,  100)
            tot += datetime.timedelta(hours=h, minutes=minutes)
        return tot

def time2sec(time):
    return time.hour * 3600 + time.minute * 60 + time.second

def sec2time(sec):
    h, rest = divmod(sec, 3600)
    m, sec = divmod(rest, 60)
    return time(h, m, sec)

def timediff(t1, t2):
    return timedelta( seconds=time2sec(t2) - time2sec(t1) )

==> jungle-1.0.2.tar.gz: jungle-1.0.2/jungle/apps/timereport/models.py <==
from django.db import models
from jungle.ref.models import WeekDay
from jungle.userprofile.models import Profile
from django.utils.translation import gettext as _
#from jungle.templatetags.jungle_tags import ugettext as _
from django.contrib.auth.models import User
from datetime import *
from django.conf import settings
from jungle.ticket.models import Project
from jungle.db import get_records
import decimal
import datetime

class TimeOutMissingTimeIn(Exception):
    pass


# class TransitManager(models.Manager):
#     def get_query_set(self):
#         from jungle.middleware.thread_local import get_current_user

#         self.user = get_current_user()
#         original = super(TransitManager, self).get_query_set()

#         if self.user and self.user.is_authenticated() and self.user.is_staff:
#             return original
#         elif self.user and self.user.is_authenticated():
#             return original.filter(user=self.user)
#         else:
#             return super(TransitManager, self).none()

STATUS_CODES = (
    (0, _('inactive')),
    (1, _('read by terminal')),
    (2, _('manually inserted')),
    (3, _('automatically inserted')),
)

DIRECTION = (
    ('0', _('No transit')),  # eg.: holiday
    ('1', _('In')),
    ('2', _('Out')),
    ('T', _('Transit')),
)
TYPES = (
    ('1', 'Transito locale presenze accessi'),
    #    ('2', 'Transito accesso remoto'),
    #    ('3', 'Transito rifiutato da controllo accessi'),
    ('4', 'Transito con causale'),  # not really used. Any transit haas a reason
    #    ('5', 'Transito con causale menu a tentina'),
    ('7', 'Consumo mensa'),
    #    ('9', 'Transito inserito da seriale'),
    #    ('A', 'Transito virtale inserito da terminale'),
    ('B', 'Prenotazione mensa'),
)
ODV_CHOICES = (
    (1, 'ingresso mattina'),
    (2, 'uscita mattina'),
    (3, 'ingresso pomeriggio'),
    (4, 'uscita pomeriggio'),
    #    (10,'extra'),
    (11, 'extra ingresso'),
    (12, 'extra uscita'),
)
##### Transit
class Transit(models.Model):

    """date_reference is in case the end of the turn passes midnight: exiting at
       2 a.m. should set date_reference the day before. That will easy all operations
    """
#    objects = TransitManager()
    objects = models.Manager()
    cron_objects = models.Manager()
    ## Campi generici
    date_create = models.DateTimeField(auto_now_add=True, null=False)
    date_last_modify = models.DateTimeField(auto_now=True, null=False)
    status = models.IntegerField(choices=STATUS_CODES, default=2)

    ## ForeignKey
    reason = models.ForeignKey('Reason', blank=True)
    user = models.ForeignKey(User, blank=True)

    ## Campi
    date_reference = models.DateField()
    date_time = models.DateTimeField(
        verbose_name=_('Datetime of transit'),
    )
    direction = models.CharField(
        max_length=1,
        verbose_name=_('Direction of transit'),
        choices=DIRECTION,
    )
    type = models.CharField(max_length=1, default=1, choices=TYPES)

    # FIXME : il profilo e' memorizzato come Integer.
    badge_num = models.CharField(max_length=12, null=True, blank=True)
    badge_reader = models.ForeignKey('BadgeReader', blank=True, null=True)
    edited_by = models.ForeignKey(User, null=True, blank=True,
                                  related_name='edited')
    profile = models.ForeignKey('TimetableProfile')
    odv = models.IntegerField(null=True, blank=True, choices=ODV_CHOICES)
    approved = models.IntegerField(null=True, blank=True)
    iter = models.ForeignKey('Iter', null=True, blank=True)

    def __cmp__(self, other):
        return cmp(self.date_time, other.date_time)

    @classmethod
    def get_worked(self, user, date):
        SQL = """SELECT date_time FROM timereport_transit t
        LEFT JOIN timereport_reason r
        ON (t.reason_id = r.code)
        WHERE t.user_id = %s AND r.worked = %s AND date_reference = %s
        ORDER BY date_time DESC"""
        transits = get_records(SQL, [user.id, 'w', date],
                               "%(date_time)s")

        if len(transits) % 2:
            return None  # Should I raise an error?
        else:
            total = timedelta()
            for i in range(0, len(transits), 2):
                total += transits[i].date_time - transits[i+1].date_time
            return total

    class Admin:
        pass

    class Meta:
        permissions = (
            ('add_others_transit', "can change other's people transits"),
        )
#        verbose_name = _('Transit')
#        verbose_name_plural = _('Transits')

    def __unicode__(self):
        return u"%s %s" % (self.user, self.date_time.strftime('%d/%m/%Y - %H:%M'))


####### BadgeReader
STATUS_CODES = (
    (0, 'inactive'),
    (1, 'active'),
)

class BadgeReader(models.Model):
    ## Campi generici
    id = models.IntegerField(primary_key=True)
    date_create = models.DateTimeField(auto_now_add=True, null=False)
    date_last_modify = models.DateTimeField(auto_now=True, null=False)
    status = models.IntegerField(choices=STATUS_CODES, default=1)
    passive = models.BooleanField(default=True, null=False)
    location = models.CharField(max_length=20)
    ip = models.CharField(max_length=15)
    user = models.CharField(max_length=15)
    password = models.CharField(max_length=15)

    def __unicode__(self):
        return unicode(self.location)

    class Admin:
        list_display = ('location', 'ip', 'user', 'password')

    class Meta:
        pass
#        verbose_name = _('Badge Reader')
#        verbose_name_plural = _('Badge Readers')

##### REASON (in italian: 'causale')
WORKED_CHOICES = (
    ('w', _('worked')),
    ('n', _('not worked'))
)
class Reason(models.Model):

    """this explains the reason of the transit. A 'default' code must be
    present. May be any locally accepted reason eg.: permit, permit due to
    medical visit... sole requirement is that worked indicate if that should
    be considered when building the total worked hours in a day.
    """
    ## Campi generici
    date_create = models.DateTimeField(auto_now_add=True, null=True)
    date_last_modify = models.DateTimeField(auto_now=True, null=True)
    status = models.IntegerField(choices=STATUS_CODES, default=1)

    ##
    code = models.CharField(max_length=7, null=True, blank=True, primary_key=True, unique=True)
    description = models.CharField(max_length=40)
    worked = models.CharField(max_length=1, choices=WORKED_CHOICES)
    column = models.CharField(max_length=7,
                              help_text=_("column in which to sum up"))

    def __unicode__(self):
        return unicode(self.description)

    class Admin:
        pass
#     class Meta:
#         verbose_name = _('Reason')
#         verbose_name_plural = _('Reasons')


class Iter(models.Model):
    ## generic fields
    date_create = models.DateTimeField(auto_now_add=True, null=False)
    date_last_modify = models.DateTimeField(auto_now=True, null=False)
    status = models.IntegerField(choices=STATUS_CODES, default=2)

    ## 'proper' fields
    date_start = models.DateField()  # mainly for 'fast_search'
    date_end = models.DateField()  # mainly for 'fast_search'
    user = models.ForeignKey(User)  # strictly unecessary and redoundant...
    description = models.CharField(max_length=100)
    date_approved1 = models.DateField(null=True)
    approved1_by = models.ForeignKey(User, null=True,
                                     related_name='approved1')

    date_approved2 = models.DateField(null=True)
    approved2_by = models.ForeignKey(User, null=True,
                                     related_name='approved2')
    reason = models.ForeignKey(Reason)

    def __unicode__(self):
        return u"%s - %s" % (self.date_start.strftime("%m/%d/%Y"),
                             self.description[:30])

    class Admin:
        pass


##### PROFILO CARTELLINO

STATUS_CODES = (
    (1, 'Active'),
    (0, 'Inactive')
)

class TimetableProfile (models.Model):

    """each employee must have a TimetableProfile that describes how the TimeRegister
    should be taken into account.
    """
    ## Campi generici
    date_create = models.DateTimeField(auto_now_add=True,)
    date_last_modify = models.DateTimeField(auto_now=True,)
    status = models.IntegerField(choices=STATUS_CODES,)
    ##
    name = models.CharField(max_length=30, unique=True)
    only_presence = models.BooleanField(default=False, null=False)
    delta_advance = models.IntegerField(verbose_name='Allowed advance flexibility in minutes')
    delta_delay = models.IntegerField(verbose_name='Allowed delay flexibility in minutes')
    hours = {}
    ProfileDayDetails = {}

    def __unicode__(self):
        return unicode(self.name)

    @classmethod
    def get_details(cls, filter={}):
        """
        get all profiles and return a dict that holds how many entries for each day
        are expected
        ProfileDayDetails = {
          '1' : {                               # profile N. 1: worker
            0 : {},                             # day 0 Sun -> no entries
            1 : {1:timedetail,3:timedetail},    # day 1 Mon -> 2 entries, 2 exit
            2 : {1:timedetail},                 # day 2 Tue -> 1 entry, 1 exit
                  ...
                  }
          '2' : ...                             # profile N. 2
          }
        """
        from time_display import MyDict
        #from inspect import isclass
        if not cls.ProfileDayDetails:
            Profiles = cls.objects.all().select_related().filter(**filter)
            for p in Profiles:
                DayEntry = MyDict()
                for i in range(1, 8):   # isoweekdays: 1 -> 7
                    details = p.timetableprofiledetail_set.filter(isoweekday=i)
                    for x in details:
                        DayEntry[i][x.odv] = x

                cls.ProfileDayDetails[p.id] = DayEntry

        return cls.ProfileDayDetails

    @classmethod
    def get_deltas(cls):
        """return a dict with deltas admitted for each profile"""
        deltas = {}
        for i in cls.objects.all():
            deltas[i.id] = (i.delta_advance, i.delta_delay)

        return deltas

    def get_day_hours(self, day):
        if isinstance(day, date):
            day = day.isoweekday()

        tot_seconds = 0
        for detail in self.get_my_details()[day].values():
            if detail:
                tot_seconds += detail.duration
        return decimal.Decimal(tot_seconds)/3600

    def get_total_working_seconds(self, isoweekday_id):
        ## in seconds since I need to compare it to datetime.timedelta

        if self.id in self.hours:
            return self.hours[self.id][isoweekday_id]

        details = self.timetableprofiledetail_set.all()
        D = {}
        for j in range(1, 8):
            D[j] = 0

        for d in details.order_by('isoweekday_id'):
            D[d.isoweekday_id] += d.duration  # hours with 2 decimals

        self.hours[self.id] = D

        return D[isoweekday_id]

    def get_my_details(self, filter={}):
        return self.get_details(filter={'id': self.id})[self.id]

    def save(self):
        self.hours = {}
        self.ProfileDayDetails = {}
        super(TimetableProfile, self).save()

    class Admin:
        pass

    class Meta:
        verbose_name = 'Orario cartellino (profilo)'
        verbose_name_plural = 'Profili Orari'


class TimetableProfileDetail (models.Model):
    ## Campi generici
    date_create = models.DateTimeField(auto_now_add=True,)
    date_last_modify = models.DateTimeField(auto_now=True,)
    status = models.IntegerField(choices=STATUS_CODES, default=1)

    ## ForeignKey
    profile = models.ForeignKey(
        TimetableProfile,
        verbose_name='Related profile',
    )
    isoweekday = models.ForeignKey(
        WeekDay,
        verbose_name='Day',
        to_field='isoweekday',
    )
    ## Campi
    time = models.TimeField(
        verbose_name='Time In',
    )
    #direction = models.IntegerField(max_length=1, null=True, blank=True,choices=((0, 'In'), (1, 'Out')))
    duration = models.IntegerField()
    ## odv in general is 0,1,2,3: 0,2 way in, 1,3 way out
    odv = models.IntegerField()

    def get_time_out(self):
        ### FIXME: does not work for nighty turns
        if not self.duration:
            return None
        secs = time2sec(self.time) + self.duration
        return sec2time(secs)

    def set_time_out(self, time):
        #import pdb; pdb.set_trace()
        ### FIXME: does not work for nighty turns
        if not self.time:
            raise TimeOutMissingTimeIn

        duration = timediff(self.time, time).seconds
        self.duration = duration

    time_out = property(get_time_out, set_time_out)

    def __unicode__(self):
        return u"%s: %s" % (self.isoweekday, self.time and self.time.strftime('%H:%M'))

    class Admin:
        pass

    class Meta:
        verbose_name = 'Dettagli Cartellino'
        verbose_name_plural = 'Dettagli Cartellino'
        unique_together = (('profile', 'isoweekday', 'odv'),)

class Final(models.Model):
    ## Campi generici
    date_create = models.DateTimeField(auto_now_add=True,)
    date_last_modify = models.DateTimeField(auto_now=True,)
    status = models.IntegerField(choices=STATUS_CODES, default=1)

    ##
    date = models.DateField()
    user = models.ForeignKey(User)
    theoric = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    worked = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    extra = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    missing = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    holiday = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    fesse = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    illness = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    cassa = models.DecimalField(max_digits=8, decimal_places=2, null=True, blank=True)
    notes = models.CharField(max_length=100)
    use_extra = models.BooleanField()

class Report(models.Model):
    #custom = ReportManager()
    ## Campi generici
    date_create = models.DateTimeField(auto_now_add=True,)
    date_last_modify = models.DateTimeField(auto_now=True,)
    status = models.IntegerField(choices=STATUS_CODES, default=1)
    ##
    user = models.ForeignKey(User)
    date = models.DateField()
    job = models.ForeignKey(Project)
    qty = models.IntegerField()
    unit = models.CharField(max_length=10)
    description = models.CharField(max_length=200)

    def hours(self):
        decimal.getcontext().prec = 3  # non funziona!
        return decimal.Decimal(self.qty)/100

    def __unicode__(self):
        return u"%s - %s - %s" % (self.user, self.date, self.job)

    def Date(self):
        return self.date.strftime("%Y/%m/%d")

    @classmethod
    def get_jobs(self, user_id, days, job_id=None):
        """return jobs 'used' in the last 'days' days"""
        date = datetime.date.today() - datetime.timedelta(days=days)
        attrs = [user_id, date]
        SQL = """SELECT DISTINCT job_id, tp.name
                 FROM timereport_report LEFT JOIN ticket_project tp
                 ON (job_id = tp.id)
                 WHERE user_id = %s AND date >= %s"""
        if job_id:
            SQL += " AND job_id = %s"
            attrs += [job_id]
        return get_records(SQL, attrs, "job: %(job_id)s")

    @classmethod
    def get_allocated(self, user_id, date):
        """return timedelta of hours in a day attributed by a user"""
        tot = datetime.timedelta()
        for e in self.objects.filter(user=user_id, date=date, qty__gt=0):
            h, minutes = divmod(e.qty,  100)
            tot += datetime.timedelta(hours=h, minutes=minutes)
        return tot

def time2sec(time):
    return time.hour * 3600 + time.minute * 60 + time.second

def sec2time(sec):
    h, rest = divmod(sec, 3600)
    m, sec = divmod(rest, 60)
    return time(h, m, sec)

def timediff(t1, t2):
    return timedelta(seconds=time2sec(t2) - time2sec(t1))
